{ self ? ./.
, pkgs ? import ./nixpkgs-2105.nix {}
, lib ? pkgs.lib
, includeStorePaths ? false
, ourpkgs ? import ./nixpkgs-ps.nix {}
, arion-src ? pkgs.fetchFromGitHub {
    owner = "tp-la";
    repo = "arion";
    rev = "hack";
    sha256 = "0wv4wbzzd926qm81h78v10wyhiaayx2jggpb1ijzk118a543sz84";
  }
}: rec {
  inherit pkgs lib ourpkgs arion-src; 
  local-grid = "${self}/morph/grid/local";
  arion-eval = args@{...}: import "${arion-src}/src/nix/eval-composition.nix" ({ inherit pkgs; } // args);
  arion = arion-eval {
    modules = [
      "${local-grid}/arion-compose.nix"
      { config._module.args = { inherit ourpkgs; } ; }
    ];
  };
  systems = pkgs.runCommand "systems" 
    { preferLocalBuild = true; allowSubstitutes = false; }
    ''
    mkdir $out
    ${lib.concatStringsSep "\n" (lib.mapAttrsToList (name: service:
      ''
      ln -s ${service.nixos.build.toplevel} $out/${lib.escapeShellArg name}
      ''
    ) arion.config.services)}'';
  bundle = pkgs.dockerTools.mergeImages (
    map (
      { imageName, imageTag, imageExe }:
      pkgs.runCommand "${baseNameOf imageName}.tar.gz"
        {
          inherit imageName;
          passthru = { inherit imageTag; };
          nativeBuildInputs = [ pkgs.pigz ];
        } "${imageExe} | pigz -nT > $out"
      ) arion.config.build.imagesToLoad
  );
  docker-yaml = pkgs.runCommand "docker-compose.yaml"
    { nativeBuildInputs = [ pkgs.gojsontoyaml ]; }
    "${pkgs.gojsontoyaml}/bin/gojsontoyaml <${arion.config.out.dockerComposeYaml} > $out"; 
  package = pkgs.runCommand "privatestorage"
     { preferLocalBuild = true; allowSubstitutes = false; }
     ''
     mkdir -p $out
     ln -sr ${bundle} $out/docker-bundle.tar.gz
     ln -sr ${docker-yaml} $out/docker-compose.yaml
     '';
   pause = pkgs.callPackage ./pause.nix {};
   morph = (import "${pkgs.morph.lib}/eval-machines.nix") {
      networkExpr = morph/grid/production/grid.nix;
   };
   morph-output = morph.machines {
      argsFile = pkgs.writeText "" (builtins.toJSON { Names = lib.attrNames morph.nodes; });
   };
}
