# See morph/grid/local/grid.nix for additional commentary.
let
  pkgs = import <nixpkgs> { };

  gridlib = import ../../lib;
  rawConfig = pkgs.lib.trivial.importJSON ./config.json;
  config = rawConfig // {
    sshUsers = import ./public-keys/users.nix;

    publicKeyPath = toString ./. + "/${rawConfig.publicKeyPath}";
    privateKeyPath = toString ./. + "/${rawConfig.privateKeyPath}";
  };

  # Configure deployment management authorization for all systems in the grid.
  grid-config = {
    imports = [
      gridlib.base
    ];

    services.private-storage.deployment = {
      authorizedKey = builtins.readFile "${config.publicKeyPath}/deploy_key.pub";
      gridName = "testing";
    };
    services.private-storage.sshUsers = config.sshUsers;
    grid = {
      inherit (config) publicKeyPath privateKeyPath; 
    };
  };

  payments = {
    imports = [
      gridlib.issuer
      gridlib.hardware-aws
      (gridlib.customize-issuer (config // {
        monitoringvpnIPv4 = "172.23.23.11";
      }))
      grid-config
    ];
  };

  storage001 = {
    imports = [
      gridlib.storage
      gridlib.hardware-aws
      ./testing001-hardware.nix
      (gridlib.customize-storage (config // {
        monitoringvpnIPv4 = "172.23.23.12";
      }))
      grid-config
    ];
    system.stateVersion = "19.03";
  };

  monitoring = {
    imports = [
      gridlib.monitoring
      gridlib.hardware-aws
      (gridlib.customize-monitoring {
        inherit hostsMap vpnClientIPs nodeExporterTargets paymentExporterTargets;
        inherit (config) domain letsEncryptAdminEmail;
        googleOAuthClientID = config.monitoringGoogleOAuthClientID;
        monitoringvpnIPv4 = "172.23.23.1";
      })
      grid-config
    ];
    system.stateVersion = "19.09";
  };

  # TBD: derive these automatically:
  paymentExporterTargets = [ "payments" ];

in {
  network = {
    description = "PrivateStorage.io Testing Grid";
    inherit pkgs;
  };
  inherit payments monitoring storage001;
}
